@@ -14,6 +14,7 @@
 #' @return character matrix with \code{n} columns.
 #' @keywords character
 #' @seealso \code{\link{str_split}} for variable number of splits
+#' @export
 #' @examples
 #' fruits <- c(
 #'   "apples and oranges and pears and bananas",
@@ -24,7 +25,7 @@
 str_split_fixed <- function(string, pattern, n) {
   if (length(string) == 0) return(matrix(character(), nrow = n, ncol = 1))
   string <- check_string(string)
-  pattern <- check_pattern(pattern)
+  pattern <- check_pattern(pattern, string)
   
   if (!is.numeric(n) || length(n) != 1) {
     stop("n should be a numeric vector of length 1")
@@ -64,6 +65,7 @@ str_split_fixed <- function(string, pattern, n) {
 #'   possible split positions.  
 #' @return a list of character vectors.
 #' @keywords character
+#' @export
 #' @seealso \code{\link{str_split_fixed}} for fixed number of splits
 #' @examples
 #' fruits <- c(
@@ -80,7 +82,7 @@ str_split_fixed <- function(string, pattern, n) {
 str_split <- function(string, pattern, n = Inf) {
   if (length(string) == 0) return(list())
   string <- check_string(string)
-  pattern <- check_pattern(pattern)
+  pattern <- check_pattern(pattern, string)
 
   if (!is.numeric(n) || length(n) != 1) {
     stop("n should be a numeric vector of length 1")
